All of lore.kernel.org
 help / color / mirror / Atom feed
From: Kevin Wolf <kwolf@redhat.com>
To: qemu-block@nongnu.org
Cc: kwolf@redhat.com, qemu-devel@nongnu.org
Subject: [Qemu-devel] [PULL 030/100] mirror: Switch mirror_cow_align() to byte-based
Date: Fri,  7 Jul 2017 19:07:45 +0200	[thread overview]
Message-ID: <1499447335-6125-31-git-send-email-kwolf@redhat.com> (raw)
In-Reply-To: <1499447335-6125-1-git-send-email-kwolf@redhat.com>

From: Eric Blake <eblake@redhat.com>

We are gradually converting to byte-based interfaces, as they are
easier to reason about than sector-based.  Convert another internal
function (no semantic change), and add mirror_clip_bytes() as a
counterpart to mirror_clip_sectors().  Some of the conversion is
a bit tricky, requiring temporaries to convert between units; it
will be cleared up in a following patch.

Signed-off-by: Eric Blake <eblake@redhat.com>
Reviewed-by: John Snow <jsnow@redhat.com>
Reviewed-by: Jeff Cody <jcody@redhat.com>
Reviewed-by: Kevin Wolf <kwolf@redhat.com>
Signed-off-by: Kevin Wolf <kwolf@redhat.com>
---
 block/mirror.c | 63 ++++++++++++++++++++++++++++++++++------------------------
 1 file changed, 37 insertions(+), 26 deletions(-)

diff --git a/block/mirror.c b/block/mirror.c
index a41ef25..0378bd2 100644
--- a/block/mirror.c
+++ b/block/mirror.c
@@ -176,6 +176,15 @@ static void mirror_read_complete(void *opaque, int ret)
     aio_context_release(blk_get_aio_context(s->common.blk));
 }
 
+/* Clip bytes relative to offset to not exceed end-of-file */
+static inline int64_t mirror_clip_bytes(MirrorBlockJob *s,
+                                        int64_t offset,
+                                        int64_t bytes)
+{
+    return MIN(bytes, s->bdev_length - offset);
+}
+
+/* Clip nb_sectors relative to sector_num to not exceed end-of-file */
 static inline int mirror_clip_sectors(MirrorBlockJob *s,
                                       int64_t sector_num,
                                       int nb_sectors)
@@ -184,44 +193,38 @@ static inline int mirror_clip_sectors(MirrorBlockJob *s,
                s->bdev_length / BDRV_SECTOR_SIZE - sector_num);
 }
 
-/* Round sector_num and/or nb_sectors to target cluster if COW is needed, and
- * return the offset of the adjusted tail sector against original. */
-static int mirror_cow_align(MirrorBlockJob *s,
-                            int64_t *sector_num,
-                            int *nb_sectors)
+/* Round offset and/or bytes to target cluster if COW is needed, and
+ * return the offset of the adjusted tail against original. */
+static int mirror_cow_align(MirrorBlockJob *s, int64_t *offset,
+                            unsigned int *bytes)
 {
     bool need_cow;
     int ret = 0;
-    int chunk_sectors = s->granularity >> BDRV_SECTOR_BITS;
-    int64_t align_sector_num = *sector_num;
-    int align_nb_sectors = *nb_sectors;
-    int max_sectors = chunk_sectors * s->max_iov;
+    int64_t align_offset = *offset;
+    unsigned int align_bytes = *bytes;
+    int max_bytes = s->granularity * s->max_iov;
 
-    need_cow = !test_bit(*sector_num / chunk_sectors, s->cow_bitmap);
-    need_cow |= !test_bit((*sector_num + *nb_sectors - 1) / chunk_sectors,
+    need_cow = !test_bit(*offset / s->granularity, s->cow_bitmap);
+    need_cow |= !test_bit((*offset + *bytes - 1) / s->granularity,
                           s->cow_bitmap);
     if (need_cow) {
-        bdrv_round_sectors_to_clusters(blk_bs(s->target), *sector_num,
-                                       *nb_sectors, &align_sector_num,
-                                       &align_nb_sectors);
+        bdrv_round_to_clusters(blk_bs(s->target), *offset, *bytes,
+                               &align_offset, &align_bytes);
     }
 
-    if (align_nb_sectors > max_sectors) {
-        align_nb_sectors = max_sectors;
+    if (align_bytes > max_bytes) {
+        align_bytes = max_bytes;
         if (need_cow) {
-            align_nb_sectors = QEMU_ALIGN_DOWN(align_nb_sectors,
-                                               s->target_cluster_size >>
-                                               BDRV_SECTOR_BITS);
+            align_bytes = QEMU_ALIGN_DOWN(align_bytes, s->target_cluster_size);
         }
     }
     /* Clipping may result in align_nb_sectors unaligned to chunk boundary, but
      * that doesn't matter because it's already the end of source image. */
-    align_nb_sectors = mirror_clip_sectors(s, align_sector_num,
-                                           align_nb_sectors);
+    align_bytes = mirror_clip_bytes(s, align_offset, align_bytes);
 
-    ret = align_sector_num + align_nb_sectors - (*sector_num + *nb_sectors);
-    *sector_num = align_sector_num;
-    *nb_sectors = align_nb_sectors;
+    ret = align_offset + align_bytes - (*offset + *bytes);
+    *offset = align_offset;
+    *bytes = align_bytes;
     assert(ret >= 0);
     return ret;
 }
@@ -257,10 +260,18 @@ static int mirror_do_read(MirrorBlockJob *s, int64_t sector_num,
     nb_sectors = MIN(s->buf_size >> BDRV_SECTOR_BITS, nb_sectors);
     nb_sectors = MIN(max_sectors, nb_sectors);
     assert(nb_sectors);
+    assert(nb_sectors < BDRV_REQUEST_MAX_SECTORS);
     ret = nb_sectors;
 
     if (s->cow_bitmap) {
-        ret += mirror_cow_align(s, &sector_num, &nb_sectors);
+        int64_t offset = sector_num * BDRV_SECTOR_SIZE;
+        unsigned int bytes = nb_sectors * BDRV_SECTOR_SIZE;
+        int gap;
+
+        gap = mirror_cow_align(s, &offset, &bytes);
+        sector_num = offset / BDRV_SECTOR_SIZE;
+        nb_sectors = bytes / BDRV_SECTOR_SIZE;
+        ret += gap / BDRV_SECTOR_SIZE;
     }
     assert(nb_sectors << BDRV_SECTOR_BITS <= s->buf_size);
     /* The sector range must meet granularity because:
@@ -184,44 +193,38 @@ static inline int mirror_clip_sectors(MirrorBlockJob *s,
         }
     }
-    /* Clipping may result in align_nb_sectors unaligned to chunk boundary, but
+    /* Clipping may result in align_bytes unaligned to chunk boundary, but
      * that doesn't matter because it's already the end of source image. */
-- 
1.8.3.1

  parent reply	other threads:[~2017-07-07 17:09 UTC|newest]

Thread overview: 104+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2017-07-07 17:07 [Qemu-devel] [PULL 000/100] Block layer patches Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 001/100] qemu-io: Don't die on second open Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 002/100] block: Guarantee that *file is set on bdrv_get_block_status() Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 003/100] block: Simplify use of BDRV_BLOCK_RAW Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 004/100] blkdebug: Support .bdrv_co_get_block_status Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 005/100] vvfat: fix qemu-img map and qemu-img convert Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 006/100] vvfat: replace tabs by 8 spaces Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 007/100] vvfat: fix typos Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 008/100] vvfat: rename useless enumeration values Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 009/100] vvfat: introduce offset_to_bootsector, offset_to_fat and offset_to_root_dir Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 010/100] vvfat: fix field names in FAT12/FAT16 and FAT32 boot sectors Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 011/100] vvfat: always create . and .. entries at first and in that order Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 012/100] vvfat: correctly create long names for non-ASCII filenames Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 013/100] vvfat: correctly create base short " Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 014/100] vvfat: correctly generate numeric-tail of short file names Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 015/100] vvfat: limit number of entries in root directory in FAT12/FAT16 Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 016/100] vvfat: handle KANJI lead byte 0xe5 Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 017/100] vvfat: change OEM name to 'MSWIN4.1' Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 018/100] qemu-img: drop -e and -6 options from the 'create' & 'convert' commands Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 019/100] blockdev: Print a warning for legacy drive options that belong to -device Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 020/100] blockjob: Track job ratelimits via bytes, not sectors Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 021/100] trace: Show blockjob actions " Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 022/100] stream: Switch stream_populate() to byte-based Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 023/100] stream: Drop reached_end for stream_complete() Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 024/100] stream: Switch stream_run() to byte-based Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 025/100] commit: Switch commit_populate() " Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 026/100] commit: Switch commit_run() " Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 027/100] mirror: Switch MirrorBlockJob " Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 028/100] mirror: Switch mirror_do_zero_or_discard() " Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 029/100] mirror: Update signature of mirror_clip_sectors() Kevin Wolf
2017-07-07 17:07 ` Kevin Wolf [this message]
2017-07-07 17:07 ` [Qemu-devel] [PULL 031/100] mirror: Switch mirror_do_read() to byte-based Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 032/100] mirror: Switch mirror_iteration() " Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 033/100] block: Drop unused bdrv_round_sectors_to_clusters() Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 034/100] backup: Switch BackupBlockJob to byte-based Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 035/100] backup: Switch block_backup.h " Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 036/100] backup: Switch backup_do_cow() " Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 037/100] backup: Switch backup_run() " Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 038/100] block: Make bdrv_is_allocated() byte-based Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 039/100] block: Minimize raw use of bds->total_sectors Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 040/100] block: Make bdrv_is_allocated_above() byte-based Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 041/100] block: expose crypto option names / defs to other drivers Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 042/100] block: add ability to set a prefix for opt names Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 043/100] qcow: document another weakness of qcow AES encryption Kevin Wolf
2017-07-07 17:07 ` [Qemu-devel] [PULL 044/100] qcow: require image size to be > 1 for new images Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 045/100] iotests: skip 042 with qcow which dosn't support zero sized images Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 046/100] iotests: skip 048 with qcow which doesn't support resize Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 047/100] block: deprecate "encryption=on" in favor of "encrypt.format=aes" Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 048/100] qcow: make encrypt_sectors encrypt in place Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 049/100] qcow: convert QCow to use QCryptoBlock for encryption Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 050/100] qcow2: make qcow2_encrypt_sectors encrypt in place Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 051/100] qcow2: convert QCow2 to use QCryptoBlock for encryption Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 052/100] qcow2: extend specification to cover LUKS encryption Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 053/100] qcow2: add support for LUKS encryption format Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 054/100] qcow2: add iotests to cover LUKS encryption support Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 055/100] iotests: enable tests 134 and 158 to work with qcow (v1) Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 056/100] block: rip out all traces of password prompting Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 057/100] block: remove all encryption handling APIs Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 058/100] block: pass option prefix down to crypto layer Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 059/100] qcow2: report encryption specific image information Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 060/100] docs: document encryption options for qcow, qcow2 and luks Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 061/100] iotests: 181 does not work for all formats Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 062/100] mirror: Fix inconsistent backing AioContext for after mirroring Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 063/100] specs/qcow2: fix bitmap granularity qemu-specific note Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 064/100] specs/qcow2: do not use wording 'bitmap header' Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 065/100] hbitmap: improve dirty iter Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 066/100] tests: add hbitmap iter test Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 067/100] block: fix bdrv_dirty_bitmap_granularity signature Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 068/100] block/dirty-bitmap: add deserialize_ones func Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 069/100] qcow2-refcount: rename inc_refcounts() and make it public Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 070/100] qcow2: add bitmaps extension Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 071/100] block/dirty-bitmap: fix comment for BlockDirtyBitmap.disabled field Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 072/100] block/dirty-bitmap: add readonly field to BdrvDirtyBitmap Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 073/100] qcow2: autoloading dirty bitmaps Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 074/100] block: refactor bdrv_reopen_commit Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 075/100] block: new bdrv_reopen_bitmaps_rw interface Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 076/100] qcow2: support .bdrv_reopen_bitmaps_rw Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 077/100] block/dirty-bitmap: add autoload field to BdrvDirtyBitmap Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 078/100] block: bdrv_close: release bitmaps after drv->bdrv_close Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 079/100] block: introduce persistent dirty bitmaps Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 080/100] block/dirty-bitmap: add bdrv_dirty_bitmap_next() Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 081/100] qcow2: add persistent dirty bitmaps support Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 082/100] qcow2: store bitmaps on reopening image as read-only Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 083/100] block: add bdrv_can_store_new_dirty_bitmap Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 084/100] qcow2: add .bdrv_can_store_new_dirty_bitmap Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 085/100] qmp: add persistent flag to block-dirty-bitmap-add Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 086/100] qmp: add autoload parameter " Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 087/100] qmp: add x-debug-block-dirty-bitmap-sha256 Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 088/100] iotests: test qcow2 persistent dirty bitmap Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 089/100] block/dirty-bitmap: add bdrv_remove_persistent_dirty_bitmap Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 090/100] qcow2: add .bdrv_remove_persistent_dirty_bitmap Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 091/100] qmp: block-dirty-bitmap-remove: remove persistent Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 092/100] block: release persistent bitmaps on inactivate Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 093/100] iotests: skip 159 & 170 with luks format Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 094/100] iotests: fix remainining tests to work with LUKS Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 095/100] iotests: reduce PBKDF iterations when testing LUKS Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 096/100] iotests: add more LUKS hash combination tests Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 097/100] iotests: chown LUKS device before qemu-io launches Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 098/100] iotests: Use absolute paths for executables Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 099/100] iotests: Add test for colon handling Kevin Wolf
2017-07-07 17:08 ` [Qemu-devel] [PULL 100/100] tests: Avoid non-portable 'echo -ARG' Kevin Wolf
2017-07-10  9:28 ` [Qemu-devel] [PULL 000/100] Block layer patches Peter Maydell
2017-07-10 11:16   ` Kevin Wolf
2017-07-11 14:35     ` [Qemu-devel] [Qemu-block] " Max Reitz

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=1499447335-6125-31-git-send-email-kwolf@redhat.com \
    --to=kwolf@redhat.com \
    --cc=qemu-block@nongnu.org \
    --cc=qemu-devel@nongnu.org \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.